#include <stdafx.h>

#include "ServoMotor.h"
#include "Serial.h"

// servo motor configuration
ServoMotor::ServoMotor(serialCL* serialCom, 
		               unsigned char boardBit):
	bit_(boardBit), serial_(serialCom), activated_(false)
{
}
	
void ServoMotor::SetBoardBit(unsigned char bit)
{
	bit_=bit;
}
void ServoMotor::SetSerialDevice(serialCL* serialCom)
{
	serial_=serialCom;
}
unsigned char ServoMotor::GetBoardBit()
{
	return bit_;
}
	
// servo motor control
int ServoMotor::SetPosition(double p)
{
	unsigned char msg[4];
	int v = SERVO_MOTOR_CL_POS_MIN+(int)(p*SERVO_MOTOR_CL_POS_DELTA);
	msg[0] = COM_REQUEST_SET_SERVO_POSITION+2*bit_;
	msg[1] = v%256;
	msg[2] = v/256;
	if(!serial_ 
		|| serial_->write(msg, 3) != SERIAL_SUCCESS) {
			return SERVO_MOTOR_ERROR;
	}
	return SERVO_MOTOR_SUCCESS;
}

int ServoMotor::GetPosition(double *p)
{
	unsigned char msg[4];
	msg[0] = COM_REQUEST_GET_SERVO_POSITION + 2*bit_;
	if(!serial_ 
		|| serial_->write(msg, 1) != SERIAL_SUCCESS) {
			return SERVO_MOTOR_ERROR;
	}
	unsigned long l=2;
	if(!serial_ 
		|| serial_->forceRead(msg, l) != SERIAL_SUCCESS) {
			return SERVO_MOTOR_ERROR;
	}
	int v = msg[0]*256+msg[1];
	*p = v/(double)SERVO_MOTOR_CL_POS_DELTA - SERVO_MOTOR_CL_POS_MIN;
	return SERVO_MOTOR_SUCCESS;
}

int ServoMotor::Activate()
{
	unsigned char msg[2];
	msg[0] = COM_REQUEST_ACTIVATE_SERVO;
	msg[1] = 1<<bit_;
	if(!serial_ 
		|| serial_->write(msg, 2) != SERIAL_SUCCESS) {
			return SERVO_MOTOR_ERROR;
	}
	activated_=true;
	return SERVO_MOTOR_SUCCESS;
}
int ServoMotor::Disactivate()
{
	unsigned char msg[2];
	msg[0] = COM_REQUEST_DISACTIVATE_SERVO;
	msg[1] = 0xff-(1<<bit_);
	if(!serial_ 
		|| serial_->write(msg, 2) != SERIAL_SUCCESS) {
			return SERVO_MOTOR_ERROR;
	}
	activated_=false;
	return SERVO_MOTOR_SUCCESS;
}

bool ServoMotor::IsActivated()
{ 
	return activated_;
}
