// Walker_Robot_Tripod.cpp: implementation of the Walker_Robot_Tripod class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "walker_texas_ranger.h"
#include "Walker_Robot_Tripod.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

int    Servo_Forward_Id=		0;
double Servo_Forward_Up=		0;  //leg up
double Servo_Forward_Ground=	0.55;
double Servo_Forward_Down=		0.9;

int    Servo_Forward_Front_Id=	7;
double Servo_Forward_Front=		0.71;
double Servo_Forward_Middle=	0.5;
double Servo_Forward_Rear=		0.33;

int    Servo_Backward_Id=		1;
double Servo_Backward_Up=		0.95;
double Servo_Backward_Ground=	0.5;
double Servo_Backward_Down=		0.05;

int    Servo_Backward_Rear_Id=	6;
double Servo_Backward_Front=	0.45;
double Servo_Backward_Middle=	0.7;
double Servo_Backward_Rear=		0.96;

int    Servo_Rotate_Id=			2;
double Servo_Rotate_Left=		0.63;
double Servo_Rotate_None=		0.61;
double Servo_Rotate_Right=		0.59;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Walker_Robot_Tripod::Walker_Robot_Tripod(serialCL* Serial) :
	serial(Serial), lastError(TRIPOD_NO_ERROR)
{
	Reset();
}

Walker_Robot_Tripod::~Walker_Robot_Tripod()
{

}

int Walker_Robot_Tripod::Reset()
{
	int i;
	// send first order to servo
	for(i=0;i<TRIPOD_NBR_SERVO;i++)
	{
		servo[i].SetSerialDevice(serial);
		servo[i].SetBoardBit(i);

		servo[i].SetPosition(0.5);
		servo[i].Disactivate();
	}
	servo[SERVO_ID_ROTATE].SetBoardBit(Servo_Rotate_Id);
	servo[SERVO_ID_UP_FORWARD].SetBoardBit(Servo_Forward_Id);
	servo[SERVO_ID_FO_FORWARD].SetBoardBit(Servo_Forward_Front_Id);
	servo[SERVO_ID_UP_BACKWARD].SetBoardBit(Servo_Backward_Id);
	servo[SERVO_ID_FO_BACKWARD].SetBoardBit(Servo_Backward_Rear_Id);
	return TRIPOD_SUCCESS;
}

int Walker_Robot_Tripod::MoveForward(double speed, int distance)
{
	int i;
	for(i=0;i<distance/30;++i){
		//prepare front leg and robot down
	servo[SERVO_ID_UP_FORWARD].SetPosition(Servo_Forward_Up);
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Ground);
	servo[SERVO_ID_UP_FORWARD].Activate();
	servo[SERVO_ID_UP_BACKWARD].Activate();
	Sleep(700);
	servo[SERVO_ID_FO_FORWARD].SetPosition(Servo_Forward_Front);
	servo[SERVO_ID_FO_FORWARD].Activate();
	Sleep(200);
	servo[SERVO_ID_UP_FORWARD].SetPosition(Servo_Forward_Ground);
	Sleep(300);
	// prepare rear leg
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Up);
	Sleep(500);
	servo[SERVO_ID_FO_BACKWARD].SetPosition(Servo_Backward_Front);
	servo[SERVO_ID_FO_BACKWARD].Activate();
	Sleep(300);
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Ground);
	Sleep(300);
	// robot up
	servo[SERVO_ID_UP_FORWARD].SetPosition(Servo_Forward_Down);
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Down);
	Sleep(700);
	// move forward
	servo[SERVO_ID_FO_BACKWARD].SetPosition(Servo_Backward_Rear);
	servo[SERVO_ID_FO_FORWARD].SetPosition(Servo_Forward_Rear);
	Sleep(700);
	}
	servo[SERVO_ID_UP_FORWARD].SetPosition(Servo_Forward_Ground);
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Ground);
	Sleep(500);
	servo[SERVO_ID_UP_FORWARD].Disactivate();
	servo[SERVO_ID_UP_BACKWARD].Disactivate();
	servo[SERVO_ID_FO_BACKWARD].Disactivate();
	servo[SERVO_ID_FO_FORWARD].Disactivate();
	Sleep(1000);

	return TRIPOD_SUCCESS;
}

int Walker_Robot_Tripod::MoveBackward(double speed, int distance)
{
	int i;
	for(i=0;i<distance/30;++i){
		//prepare front leg and robot down
	servo[SERVO_ID_UP_FORWARD].SetPosition(Servo_Forward_Up);
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Ground);
	servo[SERVO_ID_UP_FORWARD].Activate();
	servo[SERVO_ID_UP_BACKWARD].Activate();
	Sleep(700);
	servo[SERVO_ID_FO_FORWARD].SetPosition(Servo_Forward_Rear);
	servo[SERVO_ID_FO_FORWARD].Activate();
	Sleep(200);
	servo[SERVO_ID_UP_FORWARD].SetPosition(Servo_Forward_Ground);
	Sleep(300);
	// prepare rear leg
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Up);
	Sleep(500);
	servo[SERVO_ID_FO_BACKWARD].SetPosition(Servo_Backward_Rear);
	servo[SERVO_ID_FO_BACKWARD].Activate();
	Sleep(300);
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Ground);
	Sleep(300);
	// robot up
	servo[SERVO_ID_UP_FORWARD].SetPosition(Servo_Forward_Down);
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Down);
	Sleep(700);
	// move forward
	servo[SERVO_ID_FO_BACKWARD].SetPosition(Servo_Backward_Front);
	servo[SERVO_ID_FO_FORWARD].SetPosition(Servo_Forward_Front);
	Sleep(700);
	}
	servo[SERVO_ID_UP_FORWARD].SetPosition(Servo_Forward_Ground);
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Ground);
	Sleep(500);
	servo[SERVO_ID_UP_FORWARD].Disactivate();
	servo[SERVO_ID_UP_BACKWARD].Disactivate();
	servo[SERVO_ID_FO_BACKWARD].Disactivate();
	servo[SERVO_ID_FO_FORWARD].Disactivate();
	Sleep(1000);

	return TRIPOD_SUCCESS;
}

int Walker_Robot_Tripod::MoveRotate(double speed, double theta)
{
	// legs up
	servo[SERVO_ID_UP_FORWARD].SetPosition(Servo_Forward_Up);
	servo[SERVO_ID_UP_BACKWARD].SetPosition(Servo_Backward_Up);
	servo[SERVO_ID_UP_FORWARD].Activate();
	servo[SERVO_ID_UP_BACKWARD].Activate();
	Sleep(1000);
	// rotate
	if(fabs(speed)<0.2) speed=(speed>0?1:-1)*0.2;
	if(theta>0)
		servo[SERVO_ID_ROTATE].SetPosition(Servo_Rotate_None+(Servo_Rotate_Left-Servo_Rotate_None)*speed);
	else
		servo[SERVO_ID_ROTATE].SetPosition(Servo_Rotate_None+(Servo_Rotate_Right-Servo_Rotate_None)*speed);
	servo[SERVO_ID_ROTATE].Activate();
	Sleep(1000*fabs(theta));
	servo[SERVO_ID_ROTATE].Disactivate();

	return TRIPOD_SUCCESS;
}

Tripod_Error Walker_Robot_Tripod::getLastError()
{
	return lastError;
}
